package com.brilliance.swift.mx2swiftdto.bankcusta;

import com.brilliance.swift.exception.SwiftException;
import com.brilliance.swift.mx2swiftdto.AbstractMx2SwiftDtoCreator;
import com.brilliance.swift.mx2swiftdto.Mx2SwiftDtoContextIdentifier;
import com.brilliance.swift.util.DateUtil;
import com.brilliance.swift.util.StringUtil;
import com.brilliance.swift.util.XmlUtil;
import com.brilliance.swift.vo.BankToCustomerStatement;
import com.brilliance.swift.vo.common.*;
import org.dom4j.Document;

import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.ParseException;
import java.util.List;
import java.util.Map;


public class Mx2SwiftDtoBTCSCreator extends AbstractMx2SwiftDtoCreator {

    private BankToCustomerStatement bankToCustomerStatement = null;

    @Override
    public void preProcess() throws SwiftException {
        document = (Document)context.get(Mx2SwiftDtoContextIdentifier.MX_XMl_DOCUMENT, true);
        Map<String, String> parentElementMaps = XmlUtil.getParentElementMaps(document);
        appHdrParentElementName = parentElementMaps.get("AppHdr");
        grpHdrParentElementName = parentElementMaps.get("GrpHdr");
        bodyHdrParentElementName = parentElementMaps.get("Stmt");
        bankToCustomerStatement = new BankToCustomerStatement();
    }

    @Override
    public void appHeader() throws SwiftException {
        super.appHeader();
        swiftHeader.setMessageType("camt.053.001");
        String version = (String)context.get(Mx2SwiftDtoContextIdentifier.MX_VERSION, true);
        swiftHeader.setVersion(StringUtil.isEmpty(version)?"08":version);
        bankToCustomerStatement.setSwiftHeader(swiftHeader);
    }

    @Override
    public void groupHeader() throws SwiftException {
        String pgNb = getXmlNodeValue(grpHdrParentElementName,document,"GrpHdr.MsgPgntn.PgNb");
        bankToCustomerStatement.setPgNb(pgNb);
    }

    @Override
    public void messageBody() throws SwiftException {
        try {
            TransactionReference txnRef = new TransactionReference();
            txnRef.setInstructionId(getXmlNodeValue(bodyHdrParentElementName, document, "Stmt.Ntry.NtryDtls.TxDtls.Refs.InstrId"));
            txnRef.setEndToEndId(getXmlNodeValue(bodyHdrParentElementName, document, "Stmt.Ntry.NtryDtls.TxDtls.Refs.EndToEndId"));
            txnRef.setUetr(getXmlNodeValue(bodyHdrParentElementName, document, "Stmt.Ntry.NtryDtls.TxDtls.Refs.UETR"));
            bankToCustomerStatement.setTxnRef(txnRef);

            PartyAccount account = acctConvertPartyAcct(bodyHdrParentElementName, document, "Stmt.Acct");
            bankToCustomerStatement.setAccount(account);

            String elctrncSeqNb = getXmlNodeValue(bodyHdrParentElementName,document,"Stmt.ElctrncSeqNb");
            bankToCustomerStatement.setElctrncSeqNb(elctrncSeqNb);

            CashBalance openingBalance = convertCashBalanceByIndex(bodyHdrParentElementName, document, "Stmt.Bal", 1);
            bankToCustomerStatement.setOpeningBalance(openingBalance);

            CashBalance closingBalance = convertCashBalanceByIndex(bodyHdrParentElementName, document, "Stmt.Bal", 2);
            bankToCustomerStatement.setClosingBalance(closingBalance);

            List<CashBalance> cashBalanceList1 = convertCashBalanceByType(bodyHdrParentElementName, document, "Stmt.Bal", BalanceTypeCode.CLAV);
            if (cashBalanceList1.size() > 0) {
                bankToCustomerStatement.setClosingAvailableBalance(cashBalanceList1.get(0));
            }

            List<CashBalance> cashBalanceList2 = convertCashBalanceByType(bodyHdrParentElementName, document, "Stmt.Bal", BalanceTypeCode.FWAV);
            if (cashBalanceList2.size() > 0) {
                for (int i=0; i<cashBalanceList2.size(); i++) {
                    bankToCustomerStatement.addForwardAvailableBalance(cashBalanceList2.get(i));
                }
            }

            String addtlStmtInf = getXmlNodeValue(bodyHdrParentElementName,document,"Stmt.AddtlStmtInf");
            bankToCustomerStatement.setAdditionalStatementInformation(addtlStmtInf);

            int entryCount = getXmlNodeCounts(bodyHdrParentElementName, document, "Stmt.Ntry");
            if (entryCount > 0) {
                for (int i=0; i<entryCount; i++) {
                    ReportEntry reportEntry = new ReportEntry();
                    String valueDateStr = getXmlNodeValue(bodyHdrParentElementName, document, "Stmt.Ntry("+i+").ValDt.Dt");
                    if (StringUtil.isNotEmpty(valueDateStr)) {
                        reportEntry.setValueDate(DateUtil.parseDate(valueDateStr, defaultDateFormate));
                    }
                    String bookDateStr = getXmlNodeValue(bodyHdrParentElementName, document, "Stmt.Ntry("+i+").BookgDt.Dt");
                    if (StringUtil.isNotEmpty(bookDateStr)) {
                        reportEntry.setBookDate(DateUtil.parseDate(bookDateStr, defaultDateFormate));
                    }
                    String cdtDbtIndCode = getXmlNodeValue(bodyHdrParentElementName, document, "Stmt.Ntry("+i+").CdtDbtInd");
                    if (StringUtil.isNotEmpty(cdtDbtIndCode)) {
                        reportEntry.setCdtDbtCode(CdtDbtCode.valueOf(cdtDbtIndCode));
                    }
                    String rvslInd = getXmlNodeValue(bodyHdrParentElementName, document, "Stmt.Ntry("+i+").RvslInd");
                    if (StringUtil.isNotEmpty(rvslInd)) {
                        reportEntry.setReversalIndicator(Boolean.valueOf(rvslInd));
                    }
                    String ccy = getXmlNodeValue(bodyHdrParentElementName, document, "Stmt.Ntry("+i+").Amt@Ccy");
                    String amtStr = getXmlNodeValue(bodyHdrParentElementName, document, "Stmt.Ntry("+i+").Amt");
                    if (StringUtil.isNotEmpty(ccy) && StringUtil.isNotEmpty(amtStr)) {
                        CcyFormatAmount ccyFormatAmount = new CcyFormatAmount();
                        ccyFormatAmount.setCcy(ccy);
                        ccyFormatAmount.setAmt(new BigDecimal(amtStr));
                        reportEntry.setAmt(ccyFormatAmount);
                    }
                    String endToEndId = getXmlNodeValue(bodyHdrParentElementName, document, "Stmt.Ntry("+i+").NtryDtls.TxDtls.Refs.EndToEndId");
                    reportEntry.setEndToEndId(endToEndId);
                    String acctSvcrRef = getXmlNodeValue(bodyHdrParentElementName, document, "Stmt.Ntry("+i+").NtryDtls.TxDtls.Refs.AcctSvcrRef");
                    reportEntry.setAcctSvcrRef(acctSvcrRef);
                    String addtlTxInf = getXmlNodeValue(bodyHdrParentElementName, document, "Stmt.Ntry("+i+").NtryDtls.TxDtls.AddtlTxInf");
                    reportEntry.setAddtlTxInf(addtlTxInf);
                    String addtlNtryInf = getXmlNodeValue(bodyHdrParentElementName, document, "Stmt.Ntry("+i+").AddtlNtryInf");
                    reportEntry.setAddtlNtryInf(addtlNtryInf);
                    bankToCustomerStatement.addReportEntry(reportEntry);
                }
            }
        } catch (ParseException e){
            throw new SwiftException("ERROR", e.getMessage());
        }
    }
    @Override
    public void postProcess() throws SwiftException {
        try {
            context.set(Mx2SwiftDtoContextIdentifier.MX_TO_SWIFTDTO_OBJ, bankToCustomerStatement);
            String outputFilePath = (String)context.get(Mx2SwiftDtoContextIdentifier.MX_TO_SWIFTDTO_OUTPUT_FILE_PATH, true);
            if (StringUtil.isNotEmpty(outputFilePath)) {
                bankToCustomerStatement.write(new File(outputFilePath));
            }
        } catch (IOException e) {
            throw new SwiftException("ERROR", e.getMessage());
        }
    }


}
